from __future__ import annotations

try:
    from ... import Http, Lg, Time, String, Tools, Range, Random
except ImportError as e:
    if 'attempted relative import with no known parent package' in str(e):
        from bagbag import * 

import typing 

# resp = Http.Get("https://nitter.kavin.rocks/search?f=tweets&q=psychologist+a+paradox+&since=&until=&near=")

# File("tweet.html").Write(resp.Content)

# data = File("tweet.html").Read()

# x = Tools.XPath(data)

# item = x.Find("/html/body/div/div/div[2]/div[1]")

class NitterTwitterUser():
    def __init__(self) -> None:
        self.ScreenName:str = None 

        self._favoriteCount:int = None 
        self._followersCount:int = None 
        self._statusesCount:int = None 
        self._friendsCount:int = None # 当前用户去follow别人的个数

        self._name:str = None 
        # self._id:int = None 
        self._location:str = None 
        self._registerTime:int = None 
        self._description:str = None 
        self._url:str = None
        self._verified:bool = None 
        # self._listedCount:int = None 

        self.twitter:Nitter = None 
        self.infoHasBeenSet = False # 是否已经set过info
    
    def setInfoByHtml(self, html:str):
        x = Tools.XPath(html)

        self._name:str = x.Find("//a[@class='profile-card-fullname']").Text()  
        # self._id:int = None 
        self._location:str = x.Find("//div[@class='profile-location']/span[2]").Text() if  x.Find("//div[@class='profile-location']/span[2]") else None
        self._registerTime:int = Time.Strptime(x.Find("//div[@class='profile-joindate']/span").Attribute("title"), "%I:%M %p - %d %b %Y")
        self._description:str = x.Find("//div[@class='profile-bio']").Text() if x.Find("//div[@class='profile-bio']") else None
        self._url:str = x.Find("//div[@class='profile-website']/span/a").Attribute("href") if x.Find("//div[@class='profile-website']/span/a") else None
        # ipdb.set_trace()
        
        self._verified:bool = True if x.Find("//span[@class='icon-ok verified-icon' and @title='Verified account']") != None else False 
        # self._listedCount:int = None 

        self._favoriteCount = int(x.Find("//li[@class='likes']/span[2]").Text().replace(",", ""))
        self._followersCount = int(x.Find("//li[@class='followers']/span[2]").Text().replace(",", ""))
        self._friendsCount = int(x.Find("//li[@class='following']/span[2]").Text().replace(",", ""))
        self._statusesCount = int(x.Find("//li[@class='posts']/span[2]").Text().replace(",", ""))

        # Lg.Trace({
        #     "name": self._name,
        #     "location": self._location,
        #     "registerTime": self._registerTime,
        #     "description": self._description,
        #     "url": self._url,
        #     "favoriteCount": self._favoriteCount,
        #     "followersCount": self._followersCount,
        #     "friendsCount": self._friendsCount,
        #     "statusesCount": self._statusesCount,
        # })

        self.infoHasBeenSet = True
    
    def setInfo(self):
        url = self.twitter.server + "/" + self.ScreenName
        
        html = self.twitter.getHTML(url)

        self.setInfoByHtml(html)
    
    @property
    def Name(self):
        if self._name == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._name
    
    @property
    def Location(self):
        if self._location == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._location
    
    @property
    def Verified(self):
        if self._verified == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._verified
    
    @property
    def RegisterTime(self):
        if self._registerTime == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._registerTime
    
    @property
    def Description(self):
        if self._description == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._description
    
    @property
    def URL(self):
        if self._url == None and not self.infoHasBeenSet:
            self.setInfo()

        return self._url

    @property
    def FavoriteCount(self):
        if self._favoriteCount == None and not self.infoHasBeenSet:
            self.setInfo()
        
        return self._favoriteCount
    
    @property
    def FollowersCount(self):
        if self._followersCount == None and not self.infoHasBeenSet:
            self.setInfo()
        
        return self._followersCount
    
    @property
    def FriendsCount(self):
        if self._friendsCount == None and not self.infoHasBeenSet:
            self.setInfo()
        
        return self._friendsCount
    
    @property
    def StatusesCount(self):
        if self._statusesCount == None and not self.infoHasBeenSet:
            self.setInfo()
        
        return self._statusesCount
    
    def __repr__(self) -> str:
        return f"NitterTwitterUser(Name={self.Name} ScreenName={self.ScreenName} Location={self.Location} RegisterTime={self.RegisterTime} URL={self.URL} Description={self.Description} FollowersCount={self.FollowersCount} StatusesCount={self.StatusesCount} FavoriteCount={self.FavoriteCount} Verified={self.Verified})"

    def __str__(self) -> str:
        return self.__repr__()
    
    def getTimelineTweetsByHtmlByXPath(self, html:str) -> list[NitterTweet] | None:
        x = Tools.XPath(html)

        nts = []

        for idx in Range(1, 99):
            tlx = x.Find(f"//div[@class='timeline']/div[{idx}]")

            Lg.Trace("idx:", idx)

            # ipdb.set_trace()

            # weburl = 'https://twitter.com/' + x.Find(tlx + "/a").Attribute("href")
            # if x.Find(tlx + "/div/div[1]/div[1]").Attribute("class") in ["pinned", "retweet-header"]:
            #     continue 

            if tlx.Find("//div[@class='retweet-header']") != None:
                Lg.Trace("跳过转推")
                continue 

            if tlx.Find("//div[@class='pinned']") != None:
                Lg.Trace("跳过pinned")
                continue 

            if tlx.Find(f"//a[@href='/{self.ScreenName}']") != None and tlx.Find(f"//a[@href='/{self.ScreenName}']").Text() == "Load newest":
                Lg.Trace("跳过Load newest")
                continue

            if tlx.Find("//div[@class='show-more']") != None:
                break 

            if tlx.Find(f"//h2[@class='timeline-end']") != None and tlx.Find(f"//h2[@class='timeline-end']").Text() == "No more items":
                Lg.Trace("没有更多元素")
                break

            # ipdb.set_trace()

            # if tlx.Find("//a[@class='tweet-link']") == None:
            #     ipdb.set_trace()

            tid = tlx.Find("//a[@class='tweet-link']").Attribute("href").split('#')[0].split('/')[-1]
            text = tlx.Find("//div[@class='tweet-content media-body']").Text()
            time = Time.Strptime(tlx.Find("//span[@class='tweet-date']/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
            comment = tlx.Find("//div[@class='tweet-stats']/span[1]").Text().strip().replace(',', '')
            comment = int(comment) if String(comment).IsDigit() else 0

            retweet = tlx.Find("//div[@class='tweet-stats']/span[2]").Text().strip().replace(',', '')
            retweet = int(retweet) if String(retweet).IsDigit() else 0

            quota = tlx.Find("//div[@class='tweet-stats']/span[3]").Text().strip().replace(',', '')
            quota = int(quota) if String(quota).IsDigit() else 0

            like = tlx.Find("//div[@class='tweet-stats']/span[4]").Text().strip().replace(",", "")
            like = int(like) if String(like).IsDigit() else 0

            name = tlx.Find("//a[@class='fullname']").Text()

            nt = NitterTweet()
            nt.CommentCount = comment
            nt.FavoriteCount = like
            nt.ID = tid 
            nt.RetweetCount = retweet
            nt.Text = text 
            nt.Time = time 
            nt.User = self
            nt.WebURL = f"https://twitter.com/{nt.User.ScreenName}/status/{nt.ID}"

            nt.twitter = self.twitter

            nts.append(nt)

            # ipdb.set_trace()

            # Lg.Trace({
            #     "tid": tid,
            #     "username": name,
            #     "screenname": self.ScreenName,
            #     "text": text,
            #     "time": time,
            #     "comment": comment, 
            #     "retweet": retweet,
            #     "quota": quota,
            #     "like": like
            # })
            # break 

        return nts

    def getTimelineTweetsByHtml(self, html:str) -> list[NitterTweet]:
        return self.getTimelineTweetsByHtmlByXPath(html) 
    
    def getTimelineNextPageLinke(self, html:str) -> str | None:
        x = Tools.XPath(html)

        if x.Find("//div[@class='show-more']/a") == None:
            return None 
        
        return self.twitter.server + "/" + self.ScreenName + x.Find("//div[@class='show-more']/a").Attribute('href')
    
    def Tweets(self) -> typing.Iterator[NitterTweet]:
        url = self.twitter.server + "/" + self.ScreenName

        pgcount = 1
        while True:
            Lg.Trace('pgcount:', pgcount)
            html = self.twitter.getHTML(url)

            if self.infoHasBeenSet == False:
                self.setInfoByHtml(html)

            for t in self.twitter.getTweetInHtml(html):
                yield t

            url = self.getTimelineNextPageLinke(html)

            if url == None:
                break 

            pgcount += 1

class NitterTweet():
    def __init__(self) -> None:
        self.ID:int = None
        self.User:NitterTwitterUser = None 
        self.Time:int = None 
        self.Text:str = None 
        self.FavoriteCount:int = None 
        self.RetweetCount:int = None 
        self.CommentCount:int = None 
        self.WebURL:str = None 
        
        # self.in_reply_to_tweet_id:int = None
        self.twitter:Nitter = None 

    def __repr__(self) -> str:
        return f"NitterTweet(ID={self.ID} Time={self.Time} Text={self.Text} User={self.User} FavoriteCount={self.FavoriteCount} RetweetCount={self.RetweetCount} CommentCount={self.CommentCount} WebURL={self.WebURL})"
    
    def __str__(self) -> str:
        return self.__repr__()
    
    def getRepliesTweetsByHtml(self, html:str) -> list[NitterTweet]:
        x = Tools.XPath(html)

        # rtx = x.Find("//div[@class='replies']")

        nts = []

        for idx in Range(1, 99):
            tlx = x.Find(f"//div[@class='replies']/div[{idx}]/div[1]")

            Lg.Trace("idx:", idx)

            # ipdb.set_trace()

            if tlx == None:
                break 

            # ipdb.set_trace()

            # if tlx.Find("//a[@class='tweet-link']") == None:
            #     ipdb.set_trace()

            tid = tlx.Find("//a[@class='tweet-link']").Attribute("href").split('#')[0].split('/')[-1]
            text = tlx.Find("//div[@class='tweet-content media-body']").Text()
            time = Time.Strptime(tlx.Find("//span[@class='tweet-date']/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
            comment = tlx.Find("//div[@class='tweet-stats']/span[1]").Text().strip().replace(',', '')
            comment = int(comment) if String(comment).IsDigit() else 0

            retweet = tlx.Find("//div[@class='tweet-stats']/span[2]").Text().strip().replace(',', '')
            retweet = int(retweet) if String(retweet).IsDigit() else 0

            quota = tlx.Find("//div[@class='tweet-stats']/span[3]").Text().strip().replace(',', '')
            quota = int(quota) if String(quota).IsDigit() else 0

            like = tlx.Find("//div[@class='tweet-stats']/span[4]").Text().strip().replace(",", "")
            like = int(like) if String(like).IsDigit() else 0

            username = tlx.Find("//a[@class='fullname']").Text()
            screenname = tlx.Find("//a[@class='username']").Text()[1:]

            nt = NitterTweet()
            nt.CommentCount = comment
            nt.FavoriteCount = like
            nt.ID = tid 
            nt.RetweetCount = retweet
            nt.Text = text 
            nt.Time = time 
            # nt.User = self
            nt.WebURL = f"https://twitter.com/{screenname}/status/{tid}"

            nt.twitter = self.twitter

            ntu = NitterTwitterUser()
            ntu._name = username 
            ntu.ScreenName = screenname 
            ntu.twitter = self.twitter

            nt.User = ntu

            Lg.Trace({
                "tid": tid,
                "username": username,
                "screenname": screenname,
                "text": text,
                "time": time,
                "comment": comment, 
                "retweet": retweet,
                "quota": quota,
                "like": like
            })

            nts.append(nt)
        
        return nts

    def getRepliesNextPageLinkByHtml(self, html:str) -> str | None:
        x = Tools.XPath(html)

        if x.Find("//div[@class='show-more']/a") != None:
            smx = x.Find("//div[@class='show-more']/a").Attribute("href") 

            return f"{self.twitter.server}/{self.User.ScreenName}/status/{self.ID}{smx}"
        else:
            return None 
    
    def Replies(self) -> typing.Iterator[NitterTweet]:
        url = f"{self.twitter.server}/{self.User.ScreenName}/status/{self.ID}#m"

        while True:
            html = self.twitter.getHTML(url)

            for t in self.getRepliesTweetsByHtml(html):
                yield t 

            url = self.getRepliesNextPageLinkByHtml(html)

            Lg.Trace("Next URL:", url)

            if url == None:
                break 

class Nitter():
    def __init__(self, server:str=None, proxy:str=None, tor:bool=False) -> None:
        """
        proxy可以是http://或者https://以及socks5://
        如果是socks5且启用了tor需要做域名的远程解析 -> socks5h://
        """
        self.tor = tor 
        self.proxy = proxy

        if server != None:
            self.server = server.rstrip("/") 

            if not self.server.startswith("http://") and not self.server.startswith("https://"):
                self.server = 'http://' + self.server
        else:
            servers = self.getServers()
            self.selectServer(servers)
    
    def getServers(self) -> list[str]:
        servers = ['https://nitter.net']

        Lg.Trace("获取服务器列表...")
        try:
            url = 'https://raw.githubusercontent.com/wiki/zedeus/nitter/Instances.md'

            data = Http.Get(url).Content

            for i in String(data).RegexFind(r"\| *\[.+\]\((.+?)\).+?\| *:white_check_mark: *\| *:white_check_mark: *\|"):
                servers.append(i[1])
            
            Lg.Trace("获取到公网服务器列表个数:", len(servers) - 1)

            if self.tor == True:
                Lg.Trace("Tor已启用")
                for i in String(data).RegexFind(r"\| *<(http://.+?.onion)> *\| *:white_check_mark: *\|"):
                    servers.append(i[1])
                
                Lg.Trace("获取到tor网络的服务器列表个数:", len([i for i in filter(lambda x: 'onion' in x, servers)]))
        except Exception as e:
            Lg.Trace("获取服务器出错:", e)

        return servers

    def selectServer(self, servers:list[str]):
        servers = Random.Shuffle(servers)

        for server in servers:
            self.server = server.rstrip("/") 

            if not self.server.startswith("http://") and not self.server.startswith("https://"):
                self.server = 'http://' + self.server

            Lg.Trace("测试采集信息:", server)
            try:
                Lg.Trace(self.User("elonmusk"))
                Lg.Trace("使用服务器:", server)
                return 
            except Exception as e:
                Lg.Trace("出错了:", e)
                Lg.Trace("测试下一个")
        
        Lg.Trace("没有更多可以测试的服务器")

        raise Exception("没有合适的服务器")
    
    def getTweetInHtml(self, html:str) -> list[NitterTweet] | None:
        x = Tools.XPath(html)

        nts = []

        for idx in Range(1, 99):
            tlx = x.Find(f"//div[@class='timeline']/div[{idx}]")

            Lg.Trace("idx:", idx)

            # ipdb.set_trace()

            # weburl = 'https://twitter.com/' + x.Find(tlx + "/a").Attribute("href")
            # if x.Find(tlx + "/div/div[1]/div[1]").Attribute("class") in ["pinned", "retweet-header"]:
            #     continue 

            # if tlx.Find("//div[@class='retweet-header']") != None:
            #     Lg.Trace("跳过转推")
            #     continue 

            if tlx.Find("//div[@class='pinned']") != None:
                Lg.Trace("跳过pinned")
                continue 
            
            if hasattr(self, 'ScreenName'):
                if tlx.Find(f"//a[@href='/{self.ScreenName}']") != None and tlx.Find(f"//a[@href='/{self.ScreenName}']").Text() == "Load newest":
                    Lg.Trace("跳过Load newest")
                    continue
            
            if tlx.Find("//div[@class='timeline-item show-more']") and tlx.Find("//a").Text() == "Load newest":
                Lg.Trace("跳过Load newest")
                continue

            if tlx.Find("//div[@class='show-more']") != None:
                break 

            if tlx.Find(f"//h2[@class='timeline-end']") != None and tlx.Find(f"//h2[@class='timeline-end']").Text() == "No more items":
                Lg.Trace("没有更多元素")
                break

            # ipdb.set_trace()

            # if tlx.Find("//a[@class='tweet-link']") == None:
            #     ipdb.set_trace()

            tid = tlx.Find("//a[@class='tweet-link']").Attribute("href").split('#')[0].split('/')[-1]
            text = tlx.Find("//div[@class='tweet-content media-body']").Text()
            time = Time.Strptime(tlx.Find("//span[@class='tweet-date']/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
            comment = tlx.Find("//div[@class='tweet-stats']/span[1]").Text().strip().replace(',', '')
            comment = int(comment) if String(comment).IsDigit() else 0

            retweet = tlx.Find("//div[@class='tweet-stats']/span[2]").Text().strip().replace(',', '')
            retweet = int(retweet) if String(retweet).IsDigit() else 0

            quota = tlx.Find("//div[@class='tweet-stats']/span[3]").Text().strip().replace(',', '')
            quota = int(quota) if String(quota).IsDigit() else 0

            like = tlx.Find("//div[@class='tweet-stats']/span[4]").Text().strip().replace(",", "")
            like = int(like) if String(like).IsDigit() else 0

            username = tlx.Find("//a[@class='fullname']").Text()
            screenname = tlx.Find("//a[@class='username']").Text()[1:]

            nt = NitterTweet()
            nt.CommentCount = comment
            nt.FavoriteCount = like
            nt.ID = tid 
            nt.RetweetCount = retweet
            nt.Text = text 
            nt.Time = time 
            nt.twitter = self

            if type(self) == NitterTwitterUser:
                nt.User = self
            else:
                ntu = NitterTwitterUser()
                ntu._name = username 
                ntu.ScreenName = screenname 
                ntu.twitter = self

                nt.User = ntu

            nt.WebURL = f"https://twitter.com/{nt.User.ScreenName}/status/{nt.ID}"

            nts.append(nt)

            # ipdb.set_trace()

            # Lg.Trace({
            #     "tid": tid,
            #     "username": name,
            #     "screenname": self.ScreenName,
            #     "text": text,
            #     "time": time,
            #     "comment": comment, 
            #     "retweet": retweet,
            #     "quota": quota,
            #     "like": like
            # })
            # break 

        return nts
    
    def getSeachedNextPageLink(self, html:str) -> str | None:
        url = String(html).RegexFind('<div class="show-more"><a href="(.+?)">Load more</a></div>')
        if len(url) == 0:
            return None   

        return self.server + "/search" + String(url[0][1]).HTMLDecode()

    def getHTML(self, url:str) -> str:
        # headers = {
        #     "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
        #     "Connection": "keep-alive",
        #     "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
        #     "Sec-Fetch-Site": "none",
        #     "Sec-Fetch-Mode": "navigate",
        #     "Sec-Fetch-User": "?1",
        #     "Sec-Fetch-Dest": "document",
        #     "Accept-Encoding": "deflate",
        #     "Accept-Language": "en-CA,en;q=0.9,zh-CN;q=0.8,zh;q=0.7,en-GB;q=0.6,en-US;q=0.5,ur;q=0.4",
        # }
        # resp = Http.Get(url, headers=headers)

        resp = Http.Get(url, proxy=self.proxy)
        if resp.StatusCode != 200:
            raise Exception(f"HTTP状态码为{resp.StatusCode}: {url}")
        
        return resp.Content

    def Search(self, key:str) -> typing.Iterator[NitterTweet]:
        """
        注意: 不采集retweets. 因为没办法获取到转推的用户的信息. 

        The function performs a search for tweets using a given key and returns an iterator of
        NitterTweet objects, excluding retweets.
        
        :param key: The `key` parameter is a string that represents the search query. It is used to
        search for tweets on Nitter (a decentralized and privacy-focused alternative to Twitter). The
        `key` is modified by appending the `-filter:retweets` operator to exclude retweets from the
        search results
        :type key: str
        """

        key = key + " -filter:retweets"

        url = self.server + "/search?f=tweet&q=" + String(key).URLEncode()

        while True:
            html = self.getHTML(url)

            for t in self.getTweetInHtml(html):
                yield t 

            url = self.getSeachedNextPageLink(html)

            Lg.Trace("Next URL:", url)

            if url == None:
                break 
    
    def User(self, screenName:str) -> NitterTwitterUser:
        ntu = NitterTwitterUser()
        ntu.ScreenName = screenName
        ntu.twitter = self

        return ntu
    
    def getMainThreadInTweetPage(self, html:str) -> NitterTweet:
        x = Tools.XPath(html)

        mtx = x.Find("//div[@class='main-thread']") 

        username = mtx.Find("//a[@class='fullname']").Text()
        screenname = mtx.Find("//a[@class='username']").Text()[1:]
        text = mtx.Find("//div[@class='tweet-content media-body']").Text()

        time = Time.Strptime(mtx.Find("//span[@class='tweet-date']/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
        comment = mtx.Find("//div[@class='tweet-stats']/span[1]").Text().strip().replace(',', '')
        comment = int(comment) if String(comment).IsDigit() else 0

        retweet = mtx.Find("//div[@class='tweet-stats']/span[2]").Text().strip().replace(',', '')
        retweet = int(retweet) if String(retweet).IsDigit() else 0

        quota = mtx.Find("//div[@class='tweet-stats']/span[3]").Text().strip().replace(',', '')
        quota = int(quota) if String(quota).IsDigit() else 0

        like = mtx.Find("//div[@class='tweet-stats']/span[4]").Text().strip().replace(",", "")
        like = int(like) if String(like).IsDigit() else 0

        ipdb.set_trace()

        # Lg.Trace({
        #     # "tid": tid,
        #     "username": username,
        #     "screenname": screenname,
        #     "text": text,
        #     "time": time,
        #     "comment": comment, 
        #     "retweet": retweet,
        #     "quota": quota,
        #     "like": like
        # })

        nt = NitterTweet()
        nt.CommentCount = comment
        nt.FavoriteCount = like
        nt.RetweetCount = retweet
        nt.Text = text 
        nt.Time = time 
        nt.twitter = self

        ntu = NitterTwitterUser()
        ntu._name = username 
        ntu.ScreenName = screenname 
        ntu.twitter = self

        nt.User = ntu

        return nt

    def Tweet(self, tid:int) -> NitterTweet:
        url = f"{self.server}/any_username/status/{tid}#m"

        html = self.getHTML(url)

        t = self.getMainThreadInTweetPage(html)
        t.ID = tid

        return t

if __name__ == "__main__":
    # --- 联调所有
    # n = Nitter()

    # ntu = n.User("VenomApe_NFT")
    # Lg.Trace(ntu)

    # for t in ntu.Tweets():
    #     Lg.Info(Time.Strftime(t.Time), t.WebURL)

    # for t in n.Search("psychologist a paradox"):
    #     Lg.Trace("psychologist a paradox", Time.Strftime(t.Time), t.WebURL)

    # count = 0
    # for t in n.Search("musk"):
    #     Lg.Trace("musk", Time.Strftime(t.Time), t.WebURL)
    #     count += 1

    #     if count > 20:
    #         break

    # for t in n.Tweet("RDelaney", 1679898053631782924).Replies():
    #     Lg.Trace(t.ID)

    # --- 联调 tweet replies

    # n = Nitter("nitter.kavin.rocks")

    # for t in n.Tweet("RDelaney", 1679898053631782924).Replies():
    #     Lg.Trace(t.ID)

    # --- 测试 tweet replies

    # html = File("tweet.detail.single.html").Read()
    # n = Nitter("nitter.kavin.rocks")

    # t = n.getMainThreadInTweetPage(html)
    # t.ID = 1681734275589189634

    # # replies = t.getRepliesTweetsByHtml(html)
    # nextlink = t.getRepliesNextPageLinkByHtml(html)

    # Lg.Trace(nextlink)

    # --- 测试获取单条 tweet 

    # # 翻页的第一页
    # html = Http.Get("https://nitter.kavin.rocks/cz_binance/status/1681734275589189634").Content
    # File("tweet.detail.html").Write(html)

    # # 翻页的第二页
    # html = Http.Get("https://nitter.kavin.rocks/cz_binance/status/1681734275589189634?cursor=SQAAAPAMHBl2jIC9yaig3NYugICzpa-h3NYugoCxkaOoEgDxCMDSjbHJ3dYuhIC-raSi3NYuisDTjbiiLQDgsMWiotzWLiUCEhUEAAA#r").Content
    # File("tweet.detail.1.html").Write(html)

    # # 只有一页
    # html = Http.Get("https://nitter.kavin.rocks/RDelaney/status/1679898053631782924#m").Content
    # File("tweet.detail.single.html").Write(html)

    # html = File("tweet.detail.html").Read()

    n = Nitter("nitter.kavin.rocks")
    # # t = n.getMainThreadInTweetPage(html)

    t = n.Tweet(1651414032925204480)

    Lg.Trace(t)

    # --- 测试获取服务器列表

    # n = Nitter()
    # servers = n.getServers()
    # n.selectServer(servers)

    # --- 联调 timeline

    # n = Nitter("nitter.kavin.rocks")
    # n = Nitter('nitter.net')

    # ntu = n.User("VenomApe_NFT")

    # for t in ntu.Tweets():
    #     Lg.Info(Time.Strftime(t.Time), t.WebURL)

    # --- 调试 timeline 翻页链接

    # n = Nitter("nitter.kavin.rocks")

    # ntu = n.User("blahlaja")
    # html = File("user.1.html").Read()

    # link = ntu.getTimelineNextPageLinke(html)

    # Lg.Trace(link)

    # ---- 调试 NitterTwitterUser - Timeline

    # n = Nitter("nitter.kavin.rocks")

    # ntu = n.User("blahlaja")
    # html = File("user.2.html").Read()

    # ntu.getTimelineTweetsByHtml(html)

    # ---- 联调 NitterTwitterUser, 从tweets里面 - info

    # 1

    # n = Nitter("nitter.kavin.rocks")
    # for t in n.Search("psychologist a paradox"):
    #     Lg.Trace(t.ID)

    # Lg.Trace(t.User)

    # ---- 调试 NitterTwitterUser - info 

    # n = Nitter("nitter.kavin.rocks")

    # # Lg.Trace(ntu)
    # ntu = n.User("blahlaja")

    # # html = File("user.4.verified.html").Read()
    # html = File("user.html").Read()

    # # ntu = n.User("binsingha")
    # # html = File("user.1.html").Read()

    # ntu.setInfoByHtml(html)

    # Lg.Trace(ntu)

    # ---- 调试 NitterTwitterUser - info

    # 活动频繁, 好多转推
    # html = Http.Get("https://nitter.kavin.rocks/blahlaja").Content
    # File("user.html").Write(html)

    # 活动频繁
    # html = Http.Get("https://nitter.kavin.rocks/ShytoshiKusama").Content
    # File("user.2.html").Write(html)

    # 没有推文
    # html = Http.Get("https://nitter.kavin.rocks/binsingha").Content
    # File("user.1.html").Write(html)

    # html = File("user.1.html").Read()

    # ntu = NitterTwitterUser()
    # ntu.setInfoByHtml(html)

    # ---- 联调 Search 

    # n = Nitter("nitter.kavin.rocks")

    # for t in n.Search("psychologist a paradox"):
    #     Lg.Trace(t)

    # ---- 调试 Search  

    # n = Nitter("nitter.kavin.rocks")
    
    # html = File("search.retweet.html").Read()

    # for i in n.getTweetInHtml(html):
    #     Lg.Trace(i)

    # 其他

    # html = Http.Get("https://nitter.kavin.rocks/search?f=tweets&q=Si+on+prend+l%27argent+de+Musk%2C+Bezos+et+Arnault+et+qu%27on+le+redistribue+entre").Content
    # html = Http.Get("https://nitter.kavin.rocks/blahlaja").Content
    # html = Http.Get("https://nitter.kavin.rocks/cz_binance/status/1651414032925204480#m").Content
    # File("tweet.url.in.text.html").Write(html)